{% extends 'base.html' %}

{% block body %}
<header class="mdc-top-app-bar">
  <div class="mdc-top-app-bar__row">
    <section class="mdc-top-app-bar__section
                    mdc-top-app-bar__section--align-start">
      <a class="material-icons mdc-top-app-bar__navigation-icon
                mdc-icon-button" href="{% url 'program_index' %}">home</a>
      <span class="mdc-top-app-bar__title"></span>
    </section>
  </div>  
</header>
<main class="mdc-top-app-bar--fixed-adjust">
  <div class="mdc-layout-grid rp-maxwidth-grid">
    <div class="mdc-layout-grid__inner markdown-body">
      {% block error_body %} {% endblock %}
    </div>
  </div>
</main>

{% endblock %}
