{% load form_block %}

  <div class="rp-review-label mdc-layout-grid__cell
              mdc-layout-grid__cell--span-2
              mdc-layout-grid__cell--span-3-tablet
              mdc-layout-grid__cell--span-3-desktop">
  {% with label=labels|closest_label %}
    {% if label %}{{ label.display_inline }}{% else %}Address{% endif %}:
  {% endwith %}
  </div>
  <div class="rp-review-value mdc-layout-grid__cell
              mdc-layout-grid__cell--span-2
              mdc-layout-grid__cell--span-5-tablet
              mdc-layout-grid__cell--span-5-desktop">
    {% with city=block_fields.city.value %}
    {% with state=block_fields.state.value %}
    {% with pc=block_fields.postal_code.value %}
    {{ block_fields.street_address.value }}<br>
    {{ city }}{% if state %}, {{ state }}{% endif %} {{ pc }}<br>
    {# TODO: find a method that's independent of the field name #}
    {{ block_fields.country.form.instance.get__address_country_display }}
    {% endwith %}
    {% endwith %}
    {% endwith %}
  </div>
