# Generated by Django 4.0.3 on 2022-03-22 01:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('formative', '0005_alter_formblock__rank'),
    ]

    operations = [
        migrations.AlterField(
            model_name='formblock',
            name='_rank',
            field=models.IntegerField(default=0, verbose_name=''),
            preserve_default=False,
        ),
    ]
