{% load form_block %}
{% load patched_widget_tweaks %}

  <div class="rp-boolean-field mdc-form-field">
    <div class="mdc-checkbox mdc-checkbox--touch">
      {% render_field field class+="mdc-checkbox__native-control" %}
      <div class="mdc-checkbox__background">
        <svg class="mdc-checkbox__checkmark" viewBox="0 0 24 24">
          <path class="mdc-checkbox__checkmark-path" fill="none"
                d="M1.73,12.91 8.1,19.28 22.79,4.59"></path>
        </svg>
        <div class="mdc-checkbox__mixedmark"></div>
      </div>
      <div class="mdc-checkbox__ripple"></div>
    </div>
  {% with styles=form_block.form.label_class.LabelStyle %}
  {% with wlabel=labels|get_by_style:styles.WIDGET %}
    {% if wlabel %}
    <label for="{{ field.id_for_label }}">{{ wlabel.display_inline }}</label>
    {% endif %}
  {% endwith %}
  {% endwith %}
  </div>
