{% extends 'formative/base.html' %}
{% load form_signal %}

{% block page_title %}Thanks - {{ program_form.name }}{% endblock %}

{% block content %}
  <div class="mdc-layout-grid__inner">
    <div class="rp-form-top mdc-layout-grid__cell
                mdc-layout-grid__cell--span-8-tablet
                mdc-layout-grid__cell--span-8-desktop">
      {% if program_form.thanks %}
        {{ program_form.thanks }}
      {% else %}
        <p>
          Thank you for applying! We sent a confirmation to your email address.
        </p>
      {% endif %}
      
      {% form_signal program_form 'formative.signals.submission_thanks' %}
    </div>
  </div>
  
  <div class="mdc-layout-grid__inner">
    <a href="{% url 'program' program_form.program.slug %}"
       class="rp-prevpage-button mdc-icon-button material-icons"
       data-mdc-ripple-is-unbounded>
      <div class="mdc-icon-button__ripple"></div>
      chevron_left
    </a>
  </div>
{% endblock %}
