# Generated by Django 4.0.2 on 2022-02-18 18:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('formative', '0003_submissionrecord_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='form',
            name='db_slug',
            field=models.SlugField(allow_unicode=True, editable=False, max_length=30),
        ),
        migrations.AlterField(
            model_name='form',
            name='slug',
            field=models.SlugField(allow_unicode=True, max_length=30, verbose_name='identifier'),
        ),
        migrations.AlterField(
            model_name='program',
            name='db_slug',
            field=models.SlugField(allow_unicode=True, editable=False, max_length=30, unique=True),
        ),
        migrations.AlterField(
            model_name='program',
            name='name',
            field=models.CharField(max_length=64),
        ),
        migrations.AlterField(
            model_name='program',
            name='slug',
            field=models.SlugField(allow_unicode=True, max_length=30, unique=True, verbose_name='identifier'),
        ),
    ]
