{% extends 'formative/base.html' %}

{% load patched_widget_tweaks %}

{% block page_title %}{{ program_form.name }}{% endblock %}

{% block pre_layout %}
<form method="post">
{% csrf_token %}
{% endblock %}

{% block content %}
  <div class="mdc-layout-grid__inner">
    <div class="rp-form-top mdc-layout-grid__cell
                mdc-layout-grid__cell--span-8-tablet
                mdc-layout-grid__cell--span-8-desktop">
      Enter your email address to create a new application, or to recover
      one that you've already started. We will email you a link that takes
      you to the next step.
    </div>
  </div>
  
  <div class="mdc-layout-grid__inner">
  {% for field in form.visible_fields %}
    <div class="rp-layout-cell mdc-layout-grid__cell
                mdc-layout-grid__cell--span-6-tablet
                mdc-layout-grid__cell--span-6-desktop">
      {% include "./stock/textfield.html" %}
    </div>
  {% endfor %}
  </div>
  
  <div class="mdc-layout-grid__inner">
    <input class="rp-start-button mdc-button mdc-button--raised
                  mdc-layout-grid__cell mdc-layout-grid__cell--span-3"
           type="submit" value="start application"/>
  </div>
{% endblock %}

{% block post_layout %}
</form>
{% endblock %}
