<div class="mdc-layout-grid__inner">
  {% for name, field in block_fields.items %}
  <div class="rp-stock-cell mdc-layout-grid__cell mdc-layout-grid__cell--span-2
              mdc-layout-grid__cell--span-3-tablet
              mdc-layout-grid__cell--span-3-desktop">
    {% include "./textfield.html" with input_type='text' %}
  </div>
  {% endfor %}
</div>
