{% load i18n %}

<div class="form-group">
  <select class="form-control search-filter" style="width: 100%;" tabindex="-1"
          aria-hidden="true" data-name="{{ field_name }}">
    {% for choice in choices %}
      {% if choice.value %}
        <option data-name="{{ choice.name }}" value="{{ choice.value }}"
                {% if choice.selected %}selected{% endif %}>
          {{ choice.display }}
        </option>
      {% endif %}
    {% endfor %}
  </select>
</div>
