<div class="mdc-layout-grid__inner">
{% for name, field in block_fields.items %} {# assuming only one #}
  <div class="rp-stock-cell
              mdc-layout-grid__cell mdc-layout-grid__cell--span-4
              mdc-layout-grid__cell--span-{{ span|default:6 }}-tablet
              mdc-layout-grid__cell--span-{{ span|default:6 }}-desktop">
    {% include "./textfield.html" %}
  </div>
{% endfor %}
</div>
