{% extends 'formative/base.html' %}
{% load program %}

{% block page_title %}Programs{% endblock %}

{% block content %}
{% for program in programs %}
{% show_program program %}
{% endfor %}

{# TODO: show some stuff when programs is empty #}

{% endblock %}
