{% extends "admin/base_site.html" %}
{% load i18n admin_urls static jazzmin %}
{% get_jazzmin_ui_tweaks as jazzmin_ui %}

{% block extrahead %}
{{ block.super }}
{{ media }}
<script src="{% static 'admin/js/cancel.js' %}"></script>
{% endblock %}

{% block bodyclass %}
{{ block.super }} app-{{ opts.app_label }} model-{{ opts.model_name }}
{% endblock %}

{% block breadcrumbs %}
<ol class="breadcrumb float-sm-right">
<li class="breadcrumb-item">
  <a href="{% url 'admin:index' %}">
    <i class="fa fa-tachometer-alt"></i> {% trans 'Home' %}
  </a>
</li>
<li class="breadcrumb-item">
  <a href="{% url 'admin:app_list' app_label=opts.app_label %}">
    {{ opts.app_config.verbose_name }}
  </a>
</li>
{% endblock %}

{% block content_title %} {% trans 'Confirmation' %} {% endblock %}

{% block content %}

<div class="col-12">
  <div class="card card-danger card-outline">
    <div class="card-header with-border">
      <h4 class="card-title">
        {% block cardtitle %}{% endblock %}
      </h4>
    </div>
    
    <div class="card-body">
      <div id="content-main">
        <form id="{% block formid %}confform{% endblock %}" method="post">
        {% csrf_token %}
        
        {% block cardcontent %}{% endblock %}
        <div class="row">
          {% block rowcontent %}
          <div class="col-12 col-sm-9"></div>
          {% endblock %}
          <div class="col-12 col-sm-3">
            {% block rowsidecontent %}{% endblock %}
            {% block formcontrols %}{% endblock %}
          </div>
        </div>
        
        </form>
      </div>
    </div>
  </div>
</div>

{% endblock %}
