{% extends "admin/change_form.html" %}
{% load i18n admin_urls %}

{% block breadcrumbs %}
<ol class="breadcrumb float-sm-right">
<li class="breadcrumb-item">
  <a href="{% url 'admin:index' %}">
    <i class="fa fa-tachometer-alt"></i> {% trans 'Home' %}
  </a>
</li>
<li class="breadcrumb-item">
  <a href="{% url 'admin:app_list' app_label=opts.app_label %}">
    {{ opts.app_config.verbose_name }}
  </a>
</li>
<li class="breadcrumb-item">
  <a href="{% url 'admin:formative_form_changelist' %}">Forms</a>
</li>
{% if add %}
  <li class="breadcrumb-item">{% trans 'Add form' %}</li>
{% else %}
<li class="breadcrumb-item">
  <a href="{% url 'admin:formative_formblock_formlist' original.id %}">
    {{ original }}
  </a>
</li>
<li class="breadcrumb-item">settings</li>
{% endif %}
</ol>
{% endblock %}
